import { extname } from 'path';
import acorn from 'acorn-jsx';
import { walk } from 'estree-walker';
import { createFilter } from 'rollup-pluginutils';
import MagicString from 'magic-string';

// from: https://gist.github.com/DavidWells/50b891a9e012a1e748c2
function _transform(string) {
  // console.log(`in: ${string}`);
  var questionMark = string.indexOf('?');
  var colon = string.indexOf(':', questionMark);

  if (questionMark === -1 || colon === -1) {
    return string;
  }

  var condition = string.substring(0, questionMark);
  var expressions = string.substring(questionMark + 1, string.length);
  var trueExpression = null;
  var falseExpression = null;

  // console.log("expressions: " + expressions);

  // While looking in pairs, find the location where the colon occurs before the question mark.
  questionMark = expressions.indexOf('?');
  colon = expressions.indexOf(':');
  while (questionMark !== -1 && colon !== -1 && questionMark < colon) {
    questionMark = expressions.indexOf('?', questionMark + 1);
    colon = expressions.indexOf(':', colon + 1);
  }

  // console.log("\t" + "questionMark: " + questionMark);
  // console.log("\t" + "colon: " + colon);

  trueExpression = expressions.substring(0, colon);
  falseExpression = expressions.substring(colon + 1, expressions.length);

  // console.log("condition: " + condition);
  // console.log("trueExpression: " + trueExpression);
  // console.log("falseExpression: " + falseExpression);
  //
  // console.log("-");

  var result = 'if (' + condition + ') {\n' + _transform(trueExpression) + '\n} else {\n' + _transform(falseExpression) + '\n}';
  // console.log(`out: ${result}`);

  return result;
}

function plugin() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var extensions = options.extensions || ['.js'];
  var filter = createFilter(options.include, options.exclude);
  var sourceMap = options.sourceMap !== false;

  return {
    transform: function transform(code, id) {
      if (!filter(id)) {
        return null;
      }
      if (extensions.indexOf(extname(id)) === -1) {
        return null;
      }

      var ast = undefined;

      try {
        ast = acorn.parse(code, {
          ecmaVersion: 6,
          sourceType: 'module',
          plugins: {
            jsx: true
          }
        });
      } catch (err) {
        err.message += ' in ' + id;
        throw err;
      }

      var magicString = new MagicString(code);

      walk(ast, {
        enter: function enter(node) {
          if (node.type === 'ConditionalExpression') {
            if (options.extra && node.consequent.type !== 'ConditionalExpression') {
              return;
            }

            if (sourceMap) {
              magicString.addSourcemapLocation(node.start);
              magicString.addSourcemapLocation(node.end);
            }

            console.log('\nfile: ' + id);
            var converted = _transform(code.slice(node.start, node.end));
            magicString.overwrite(node.start, node.end, converted);
            return;
          }
        }
      });

      var resultCode = magicString.toString();
      var map = sourceMap ? magicString.generateMap() : null;

      return {
        code: resultCode,
        map: map
      };
    }
  };
}

export default plugin;